import type { RuleItem } from 'async-validator';
import type { Component, Ref, StyleValue } from 'vue';
import type { LabelPoistion } from './inject';
export type ObjectModel = Record<string, any>;
export declare enum FormItemType {
    /**
     * 自定义组件
     */
    CUSTOM = "CUSTOM",
    /**
     * 预设组件，通过provide/inject 注册
     */
    PREDEFINE = "PREDEFINE"
}
/**
 * 表单项基础数据
 */
export type BaseFormItemData<T extends FormItemType> = {
    /**
     * 类型
     */
    type: T | `${T}`;
    /**
     * 唯一key
     */
    key: string;
    /**
     * 初始值
     */
    initial?: number | string | Function;
    /**
     * 是否必填
     */
    rules?: RuleItem | RuleItem[];
    /**
     * label
     */
    label?: string;
    /**
     * label
     */
    placeholder?: string;
    /**
     * FormItem 样式
     */
    itemClass?: string;
    itemStyle?: StyleValue;
    labelPosition?: LabelPoistion;
    disabled?: boolean;
    modelValueKey?: string;
    /**
     * [modelValueKey]: your model key
     */
    modelKeyMap?: Record<string, string>;
    attrs?: Record<string, any>;
};
export type FormItemData = CustomCompData | PreDefineCompData;
/**
 * 表单提交结果
 */
export type FormSchema = FormItemData[];
export interface CustomCompData<C = any> extends BaseFormItemData<FormItemType.CUSTOM> {
    component: C;
    props?: C extends Component<infer P> ? P extends {
        $props: infer Props;
    } ? MaybeRefOpions<Omit<Props, 'modelValue'>> : any : any;
    slots?: SlotType<C>;
    slot?: DefaultSlot<SlotType<C>>;
}
export type MaybeRefOpions<T> = {
    [P in keyof T]?: T[P] | Ref<T[P]>;
};
export interface PreDefineCompData<Alias extends string = any, C = any> extends BaseFormItemData<FormItemType.PREDEFINE> {
    component: C;
    props?: C extends Component<infer P> ? P extends {
        $props: infer Props;
    } ? MaybeRefOpions<Omit<Props, 'modelValue'>> : any : any;
    slots?: SlotType<C>;
    slot?: DefaultSlot<SlotType<C>>;
    alias: Alias;
}
type SlotType<C> = C extends Component<infer P> ? P extends {
    $slots: infer Slots;
} ? Convert<Required<Slots>> : any : any;
type Convert<M> = {
    [K in keyof M]?: M[K] extends (...args: any[]) => any ? Component<Parameters<M[K]>[0]> | string : Component | string;
};
type DefaultSlot<S> = S extends {
    [x: string]: any;
} ? S['default'] : never;
export {};
