import { ref } from 'vue'
import type ItemRender from './ItemRender.vue'

export const useSmartForm = () => {
  const items = ref<Record<string, InstanceType<typeof ItemRender>>>({})
  const setItems = (e: any, p: string) => {
    items.value[p] = e
  }

  /**
   * 获取结果
   * @param dirty 是否只获取修改过的 @default false
   */
  const getResult = async (dirty = false) => {
    const errors: Error[] = []
    const res = {}
    for (const item of Object.entries(items.value)) {
      const [k, e] = item
      if (dirty && !e.model.$dirty) return res
      const r = await e.getResult()
      if (e.model.$anyInvalid) {
        //@ts-ignore
        errors.push(new Error(e.model.$errors[0]))
      }

      Object.assign(res, r)
    }

    if (errors.length) {
      throw errors[0]
    }
    return res
  }

  /**
   * 重制dirtry
   */
  const reset = () => {
    Object.values(items.value).forEach((e) => e.reset())
  }

  return {
    items,
    setItems,
    getResult,
    reset,
  }
}
